#include "sdlskk.h"
#include "sdlskk_internal.h"
#include <string.h>
#include <stdio.h>

static void SDLSKK_Map_add( SDLSKK_Map *map, SDLSKK_Char * key, SDLSKK_Char *val )
{
  SDLSKK_StrList *list;
  
  list = SDLSKK_Map_get( map, key );
  if( list == NULL ){
    list = SDLSKK_StrList_new();
    SDLSKK_Map_set( map, key, list );
  }
  SDLSKK_StrList_push( list, val );
  return ;
}

#define BUF_SIZE 1024
static void parse_skk_jisyo_line( SDLSKK_Map* map, char* line )
{
  int pos;
  SDLSKK_Char* yomi;
  SDLSKK_Char* kanji;
  
  if( line[0] == ';' || line[0] == ' ' || line[0] == '\n' || line[0] == '\0' )
    return;

  pos = 0;
  yomi = SDLSKK_cstr_tokenize( line, '/', &pos );
  SDLSKK_Str_chop( yomi );

  for(;;){
    kanji = SDLSKK_cstr_tokenize( line, '/', &pos );
    if( kanji[0]==0 ){
      free( kanji );
      return;
    }
    SDLSKK_Map_add( map, yomi, kanji );
  }
  
}

SDLSKK_Dictionary* SDLSKK_Dict_new(char *filename)
{  
  SDLSKK_Dictionary* dict;

  dict = malloc( sizeof(SDLSKK_Dictionary) );
  dict->map = SDLSKK_Map_new( DICT_TABLE_SIZE );

  SDLSKK_Dict_load( dict, filename );
  return dict;
}

int SDLSKK_Dict_load( SDLSKK_Dictionary* dict, char* filename )
{
  FILE* fp;
  char buf[2048];
  
  fp = fopen( filename, "r" );
  if( fp==NULL )
    return 0;

  while( fgets( buf, sizeof(buf), fp ) != NULL ){
    parse_skk_jisyo_line( dict->map, buf );    
  }
  fclose( fp );
  return 1;
}

void SDLSKK_Dict_delete( SDLSKK_Dictionary* dict )
{
  SDLSKK_Map_delete( dict->map );
}

void SDLSKK_Dict_delete_all( SDLSKK_Dictionary* dict )
{
  SDLSKK_Map_delete( dict->map );
}

SDLSKK_DictionaryItem* SDLSKK_Dict_get( SDLSKK_Dictionary* dict,
					SDLSKK_Char *hiragana )
{
  SDLSKK_StrList *vals = SDLSKK_Map_get( dict->map, hiragana );
  SDLSKK_DictionaryItem* item;
  if( vals == NULL )
    return NULL;
  
  item = malloc( sizeof(SDLSKK_DictionaryItem) );
  item->dict = dict;
  item->hiragana = hiragana;
  item->kanji = vals;
  item->cur = item->kanji->begin->next;
  
  return item;
}

SDLSKK_Char* SDLSKK_DictItem_get_current( SDLSKK_DictionaryItem* item )
{
  return item->cur->val;
}

void SDLSKK_DictItem_next( SDLSKK_DictionaryItem* item )
{
  if( SDLSKK_DictItem_is_end( item ) )
    return;
  item->cur = item->cur->next;
}

void SDLSKK_DictItem_prev( SDLSKK_DictionaryItem* item )
{
  if( SDLSKK_DictItem_is_begin( item ) )
    return;
  item->cur = item->cur->prev;
}
void SDLSKK_DictItem_reset( SDLSKK_DictionaryItem* item )
{
  item->cur = item->kanji->begin->next;
}

void SDLSKK_DictItem_delete( SDLSKK_DictionaryItem* item )
{
  free( item );
}

int SDLSKK_DictItem_is_end( SDLSKK_DictionaryItem* item )
{
  return item->cur->next == item->kanji->end;
}

int SDLSKK_DictItem_is_begin( SDLSKK_DictionaryItem* item )
{
  return item->cur->prev == item->kanji->begin;
}

void SDLSKK_DictItem_commit( SDLSKK_DictionaryItem* item )
{
  item->cur->prev->next = item->cur->next;
  item->cur->next->prev = item->cur->prev;

  item->cur->prev = item->kanji->begin;
  item->cur->next = item->kanji->begin->next;
  item->kanji->begin->next->prev = item->cur;
  item->kanji->begin->next = item->cur;
}
